package X3dForWebAuthors.Chapter15Metadata;

/*
Copyright (c) 1995-2022 held by the author(s).  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer
      in the documentation and/or other materials provided with the
      distribution.
    * Neither the name of the Web3D Consortium (https://www.web3d.org)
      nor the names of its contributors may be used to endorse or
      promote products derived from this software without specific
      prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

import org.web3d.x3d.jsail.Core.*;
import org.web3d.x3d.jsail.EnvironmentalEffects.*;
import org.web3d.x3d.jsail.fields.*;
import org.web3d.x3d.jsail.Navigation.*;
import org.web3d.x3d.jsail.Networking.*;
import org.web3d.x3d.jsail.Shape.*;
import org.web3d.x3d.jsail.Text.*;

// Javadoc annotations follow, see below for Java source code.
/**
 * <p> Simple examples of meta statements, comments, WorldInfo node and typed metadata nodes. </p>
 <p> Related links: <a href="../../../Chapter15Metadata/MetadataNodeExamplesX3D4.java">MetadataNodeExamplesX3D4.java</a> source, <a href="../../../Chapter15Metadata/MetadataNodeExamplesX3D4Index.html" target="_top">MetadataNodeExamplesX3D4 catalog page</a>, <a href="https://www.web3d.org/x3d/content/examples/X3dResources.html" target="_blank">X3D Resources</a>, <a href="https://www.web3d.org/x3d/content/examples/X3dSceneAuthoringHints.html" target="_blank">X3D Scene Authoring Hints</a>, and <a href="https://www.web3d.org/x3d/content/X3dTooltips.html" target="_blank">X3D Tooltips</a>. </p>
	<table style="color:black; border:0px solid; border-spacing:10px 0px;">
        <caption>Scene Meta Information</caption>
		<tr style="background-color:silver; border-color:silver;">
			<td style="text-align:center; padding:10px 0px;"><i>meta tags</i></td>
			<td style="text-align:left;   padding:10px 0px;">&nbsp; Document Metadata </td>
		</tr>

		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> title </i> </td>
			<td> <a href="../../../Chapter15Metadata/MetadataNodeExamplesX3D4.x3d">MetadataNodeExamplesX3D4.x3d</a> </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> description </i> </td>
			<td> Simple examples of meta statements, comments, WorldInfo node and typed metadata nodes. </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> creator </i> </td>
			<td> Don Brutzman </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> created </i> </td>
			<td> 30 September 2011 </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> modified </i> </td>
			<td> 24 April 2022 </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> reference </i> </td>
			<td> Original name MetadataNodeExamples.x3d, renamed since handling of metadata containerField defaults changed in X3D4. </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> reference </i> </td>
			<td> <a href="../../../Chapter15Metadata/MetadataNodeExamplesX3D3.x3d">MetadataNodeExamplesX3D3.x3d</a> </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> info </i> </td>
			<td> Information in head and meta elements is retained after a scene is parsed and loaded, and can be referenced via the Scene Access Interface (SAI) or Document Object Model (DOM) </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> info </i> </td>
			<td> Authoring note: MetadataBoolean requires X3D version 4.0, as used in this example scene </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> reference </i> </td>
			<td> The following referenced chapter is published online but was not included in the printed book. </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> reference </i> </td>
			<td> <a href="../../../Chapter15Metadata/Chapter15MetadataInformation.html">Chapter15MetadataInformation.html</a> </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> reference </i> </td>
			<td> <a href="https://x3dgraphics.com/chapters/Chapter15MetadataInformation.html" target="_blank">https://x3dgraphics.com/chapters/Chapter15MetadataInformation.html</a> </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> reference </i> </td>
			<td> <a href="https://X3dGraphics.com" target="_blank">https://X3dGraphics.com</a> </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> reference </i> </td>
			<td> <a href="https://www.web3d.org/x3d/content/examples/X3dResources.html" target="_blank">https://www.web3d.org/x3d/content/examples/X3dResources.html</a> </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> rights </i> </td>
			<td> Copyright 2006, Daly Realism and Don Brutzman </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> subject </i> </td>
			<td> X3D book, X3D graphics, X3D-Edit, <a href="http://www.x3dGraphics.com" target="_blank">http://www.x3dGraphics.com</a> </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> identifier </i> </td>
			<td> <a href="https://X3dGraphics.com/examples/X3dForWebAuthors/Chapter15Metadata/MetadataNodeExamplesX3D4.x3d" target="_blank">https://X3dGraphics.com/examples/X3dForWebAuthors/Chapter15Metadata/MetadataNodeExamplesX3D4.x3d</a> </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> generator </i> </td>
			<td> X3D-Edit 3.3, <a href="https://savage.nps.edu/X3D-Edit" target="_blank">https://savage.nps.edu/X3D-Edit</a> </td>
		</tr>
		<tr>
			<td style="text-align:right; vertical-align: text-top;"> <i> license </i> </td>
			<td> <a href="../../../Chapter15Metadata/../license.html">../license.html</a> </td>
		</tr>
		<tr style="background-color:silver; border-color:silver;">
			<td style="text-align:center;" colspan="2">  &nbsp; </td>
		</tr>
	</table>

	<p>
		This program uses the
		<a href="https://www.web3d.org/specifications/java/X3DJSAIL.html" target="_blank">X3D Java Scene Access Interface Library (X3DJSAIL)</a>.
		It has been produced using the 
		<a href="https://www.web3d.org/x3d/stylesheets/X3dToJava.xslt" target="_blank">X3dToJava.xslt</a>
		stylesheet to create Java source code from an <code>.x3d</code> model.
	</p>

	* @author Don Brutzman
 */

public class MetadataNodeExamplesX3D4
{
	/** Default constructor to create this object. */
	public MetadataNodeExamplesX3D4 ()
	{
	  initialize();
	}

	/** Create and initialize the X3D model for this object. */
	public final void initialize()
	{
  x3dModel = new X3D().setProfile(X3D.PROFILE_IMMERSIVE).setVersion(X3D.VERSION_4_0)
  .setHead(new head()
    .addMeta(new meta().setName(meta.NAME_TITLE      ).setContent("MetadataNodeExamplesX3D4.x3d"))
    .addMeta(new meta().setName(meta.NAME_DESCRIPTION).setContent("Simple examples of meta statements, comments, WorldInfo node and typed metadata nodes."))
    .addMeta(new meta().setName(meta.NAME_CREATOR    ).setContent("Don Brutzman"))
    .addMeta(new meta().setName(meta.NAME_CREATED    ).setContent("30 September 2011"))
    .addMeta(new meta().setName(meta.NAME_MODIFIED   ).setContent("24 April 2022"))
    .addMeta(new meta().setName(meta.NAME_REFERENCE  ).setContent("Original name MetadataNodeExamples.x3d, renamed since handling of metadata containerField defaults changed in X3D4."))
    .addMeta(new meta().setName(meta.NAME_REFERENCE  ).setContent("MetadataNodeExamplesX3D3.x3d"))
    .addMeta(new meta().setName(meta.NAME_INFO       ).setContent("Information in head and meta elements is retained after a scene is parsed and loaded, and can be referenced via the Scene Access Interface (SAI) or Document Object Model (DOM)"))
    .addMeta(new meta().setName(meta.NAME_INFO       ).setContent("Authoring note: MetadataBoolean requires X3D version 4.0, as used in this example scene"))
    .addMeta(new meta().setName(meta.NAME_REFERENCE  ).setContent("The following referenced chapter is published online but was not included in the printed book."))
    .addMeta(new meta().setName(meta.NAME_REFERENCE  ).setContent("Chapter15MetadataInformation.html"))
    .addMeta(new meta().setName(meta.NAME_REFERENCE  ).setContent("https://x3dgraphics.com/chapters/Chapter15MetadataInformation.html"))
    .addMeta(new meta().setName(meta.NAME_REFERENCE  ).setContent("https://X3dGraphics.com"))
    .addMeta(new meta().setName(meta.NAME_REFERENCE  ).setContent("https://www.web3d.org/x3d/content/examples/X3dResources.html"))
    .addMeta(new meta().setName(meta.NAME_RIGHTS     ).setContent("Copyright 2006, Daly Realism and Don Brutzman"))
    .addMeta(new meta().setName(meta.NAME_SUBJECT    ).setContent("X3D book, X3D graphics, X3D-Edit, http://www.x3dGraphics.com"))
    .addMeta(new meta().setName(meta.NAME_IDENTIFIER ).setContent("https://X3dGraphics.com/examples/X3dForWebAuthors/Chapter15Metadata/MetadataNodeExamplesX3D4.x3d"))
    .addMeta(new meta().setName(meta.NAME_GENERATOR  ).setContent("X3D-Edit 3.3, https://savage.nps.edu/X3D-Edit"))
    .addMeta(new meta().setName(meta.NAME_LICENSE    ).setContent("../license.html")))
  .setScene(new Scene()
    .addComments(" May 2017: X3D4 Architecture Specification clarification allows Metadata nodes at top level of Scene graph. ")
    .addMetadata(new MetadataString().setName("TestRootMetadataNode").setValue(new String[] {"Test satisfactory.","Note that all SFString values must be quoted."}))
    .addComments(" Comments can appear between nodes (XML elements) but are not retained after a scene is parsed, loaded and playing ")
    .addChild(new WorldInfo("MyWorldInfo").setInfo(new String[] {"Metadata strings","can go here","as string array values"}).setTitle("MetadataNodeExamplesX3D4.x3d Example Scene")
      .addComments(" Structured information can go in Metadata nodes. Note that any X3D node can contain a single Metadata node with containerField='metadata' ")
      .setMetadata(new MetadataSet("MyMetadataSetNode").setName("Metadata_name").setReference("SomeReferenceStandard 1.2.3c")
        .setMetadata(new MetadataBoolean("MyMetadataBooleanNode").setName("Coin Flip").setReference("MetadataBoolean allowed in X3D v3.3 and later").setValue(new boolean[] {true,false,true,false}))
        .setMetadata(new MetadataDouble("MyMetadataDoubleNode").setName("Metadata_name").setReference("SomeReferenceStandard 1.2.3c").setValue(new double[] {3.141592658,2.71812181}))
        .setMetadata(new MetadataFloat("MyMetadataFloatNode").setName("Metadata_name").setReference("SomeReferenceStandard 1.2.3c").setValue(new double[] {9.8,6.023e+23}))
        .setMetadata(new MetadataInteger("MyMetadataIntegerNode").setName("Metadata_name").setReference("SomeReferenceStandard 1.2.3c").setValue(new int[] {6,28,496}))
        .setMetadata(new MetadataString("MyMetadataStringNode").setName("Metadata_name").setReference("SomeReferenceStandard 1.2.3c").setValue(new String[] {"Part 27","P27","p27"}))
        .setMetadata(new MetadataSet("NestedMetadataSetNode").setName("TestNestedMetadataSetNodes")
          .addComments(" MetadataSet can also contain multiple Metadata nodes with containerField='value' ")
          .setMetadata(new MetadataString().setUSE("MyMetadataStringNode"))
          .setMetadata(new MetadataInteger().setUSE("MyMetadataIntegerNode"))
          .setMetadata(new MetadataFloat().setUSE("MyMetadataFloatNode"))
          .setMetadata(new MetadataFloat().setName("coefficients").setValue(new double[] {3.141592653,2.7128,1.0,0.0})))))
    .addChild(new Background().setSkyColor(new MFColor(new double[] {0.0,0.439216,0.760784})))
    .addChild(new Viewpoint().setDescription("View scene source to see metadata examples").setPosition(0.0,0.0,9.0))
    .addChild(new Anchor().setDescription("Load scene index page").setParameter(new String[] {"target=_blank"}).setUrl(new String[] {"MetadataNodeExamplesIndex.html","https://X3dGraphics.com/examples/X3dForWebAuthors/Chapter15Metadata/MetadataNodeExamplesIndex.html"})
      .addChild(new Shape()
        .setGeometry(new Text().setString(new String[] {"View scene source","to see","metadata examples"})
          .setFontStyle(new FontStyle().setJustify(FontStyle.JUSTIFY_MIDDLE_MIDDLE)))
        .setAppearance(new Appearance()
          .setMaterial(new Material().setDiffuseColor(1.0,0.992157,0.039216))))));
    }
	// end of initialize() method

	/** The initialized model object, created within initialize() method. */
	private X3D x3dModel;

	/** 
	 * Provide a 
	 * <a href="https://dzone.com/articles/java-copy-shallow-vs-deep-in-which-you-will-swim" target="_blank">shallow copy</a>
	 * of the X3D model.
	 * @see <a href="https://www.web3d.org/specifications/java/javadoc/org/web3d/x3d/jsail/Core/X3D.html">X3D</a>
	 * @return MetadataNodeExamplesX3D4 model
	 */
	public X3D getX3dModel()
	{	  
		return x3dModel;
	}
	   
    /** 
	 * Default main() method provided for test purposes, uses CommandLine to set global ConfigurationProperties for this object.
     * @param args array of input parameters, provided as arguments
	 * @see <a href="https://www.web3d.org/specifications/java/javadoc/org/web3d/x3d/jsail/Core/X3D.html#handleArguments-java.lang.String:A-">X3D.handleArguments(args)</a>
	 * @see <a href="https://www.web3d.org/specifications/java/javadoc/org/web3d/x3d/jsail/Core/X3D.html#validationReport--">X3D.validationReport()</a>
     * @see <a href="https://www.web3d.org/specifications/java/javadoc/org/web3d/x3d/jsail/CommandLine.html">CommandLine</a>
     * @see <a href="https://www.web3d.org/specifications/java/javadoc/org/web3d/x3d/jsail/CommandLine.html#USAGE">CommandLine.USAGE</a>
     * @see <a href="https://www.web3d.org/specifications/java/javadoc/org/web3d/x3d/jsail/ConfigurationProperties.html">ConfigurationProperties</a>
     */
    public static void main(String args[])
    {
        X3D thisExampleX3dModel = new MetadataNodeExamplesX3D4().getX3dModel();

		boolean hasArguments = (args != null) && (args.length > 0);
		boolean validate = true; // default
		boolean argumentsLoadNewModel = false;
		String  fileName = new String();

		if (args != null)
		{
			for (String arg : args)
			{
				if (arg.toLowerCase().startsWith("-v") || arg.toLowerCase().contains("validate"))
				{
					validate = true; // making sure
				}
				if (arg.toLowerCase().endsWith(X3D.FILE_EXTENSION_X3D) ||
					arg.toLowerCase().endsWith(X3D.FILE_EXTENSION_CLASSICVRML) ||
					arg.toLowerCase().endsWith(X3D.FILE_EXTENSION_X3DB) ||
					arg.toLowerCase().endsWith(X3D.FILE_EXTENSION_VRML97) ||
					arg.toLowerCase().endsWith(X3D.FILE_EXTENSION_EXI) ||
					arg.toLowerCase().endsWith(X3D.FILE_EXTENSION_GZIP) ||
					arg.toLowerCase().endsWith(X3D.FILE_EXTENSION_ZIP) ||
					arg.toLowerCase().endsWith(X3D.FILE_EXTENSION_HTML) ||
					arg.toLowerCase().endsWith(X3D.FILE_EXTENSION_XHTML))
				{
					argumentsLoadNewModel = true;
					fileName = arg;
				}
			}
		}
		if      (argumentsLoadNewModel)
			System.out.println("WARNING: \"X3dForWebAuthors.Chapter15Metadata.MetadataNodeExamplesX3D4\" model invocation is attempting to load file \"" + fileName + "\" instead of simply validating itself... file loading ignored.");
		else if (hasArguments) // if no arguments provided, this method produces usage warning
			thisExampleX3dModel.handleArguments(args);

		if (validate)
		{
			System.out.print("Java program \"X3dForWebAuthors.Chapter15Metadata.MetadataNodeExamplesX3D4\" self-validation test results: ");
			String validationResults = thisExampleX3dModel.validationReport();
            if (validationResults.startsWith("\n"))
                System.out.println();
			System.out.println(validationResults.trim());
		}
    }
}
