####################################################################################################
#
# Now available: developmental python x3d.py package on PyPI for import.
#   This approach simplifies Python X3D deployment and use.
#   https://pypi.org/project/x3d
#
# Installation:
#       pip install x3d
# or
#       python -m pip install x3d
#
# Developer options for loading x3d package:
#
#    from x3d import *  # preferred approach, terser source that avoids x3d.* class prefixes
#
# or
#    import x3d         # traditional way to subclass x3d package, all classes require x3d.* prefix,
#                       # but python source is very verbose, for example x3d.Material x3d.Shape etc.
#                       # X3dToPython.xslt stylesheet insertPackagePrefix=true supports this option.
#
####################################################################################################

from x3d import *

newModel=X3D(profile='Immersive',version='3.3',
  head=head(
    children=[
    meta(content='HalfMoonPrototype.x3d',name='title'),
    meta(content='Prototype for a group of HalfMoon fish in the Kelp forest.',name='description'),
    meta(content='Ildeniz Duman',name='creator'),
    meta(content='Scott Tufts',name='translator'),
    meta(content='1 June 1998',name='created'),
    meta(content='22 February 2022',name='modified'),
    meta(content='Additional PositionInterpolator present but disabled.',name='warning'),
    meta(content='Wiggle causes texture flashing. TODO: move animation to example file.',name='warning'),
    meta(content='https://X3dGraphics.com/examples/X3dForWebAuthors/KelpForestExhibit/HalfMoonPrototype.x3d',name='identifier'),
    meta(content='X3D-Edit 3.3, https://savage.nps.edu/X3D-Edit',name='generator'),
    meta(content='../license.html',name='license')]),
  Scene=Scene(
    children=[
    WorldInfo(title='HalfMoonPrototype.x3d'),
    ProtoDeclare(appinfo='HalfMoon fish',name='HalfMoon',
      ProtoInterface=ProtoInterface(
        field=[
        field(accessType='inputOutput',name='skinTextureUrl',type='MFString',value=["HalfMoonScalesPatch.jpg","https://X3dGraphics.com/examples/X3dForWebAuthors/KelpForestExhibit/HalfMoonScalesPatch.jpg"]),
        field(accessType='inputOutput',name='fishScale',type='SFVec3f',value=(0.3,0.25,0.25)),
        field(accessType='inputOutput',name='fishColor',type='SFColor',value=(0.7,0.7,0.7))]),
      ProtoBody=ProtoBody(
        children=[
        Transform(DEF='MyHalfMoon',
          IS=IS(
            connect=[
            connect(nodeField='scale',protoField='fishScale')]),
          children=[
          Shape(
            appearance=Appearance(
              material=Material(DEF='FishColor',
                IS=IS(
                  connect=[
                  connect(nodeField='diffuseColor',protoField='fishColor')])),
              texture=ImageTexture(DEF='FishTexture',
                IS=IS(
                  connect=[
                  connect(nodeField='url',protoField='skinTextureUrl')]))),
            geometry=Extrusion(DEF='Fish_Shape',creaseAngle=1.57,crossSection=[(-0.1,1.0),(0.1,1),(0.5,0.5),(0.5,-0.5),(0.1,-1),(-0.1,-1),(-0.5,-0.5),(-0.5,0.5),(-0.1,1)],scale=[(0.03,0.03),(0.3,0.3),(0.2,0.4),(0.1,0.5),(0.1,0.15),(0.03,0.4)],spine=[(0,1.5,0),(0,1.2,0),(0,0.5,0),(0,0.3,0),(0,0,0),(0,-0.3,0)])),
          Shape(
            appearance=Appearance(DEF='Fin_Skin',
              material=Material(diffuseColor=(0.5,0.5,0.5),transparency=0.2))),
          Transform(translation=(1.37,0.1,0.1),
            children=[
            Shape(DEF='Eye',
              appearance=Appearance(
                material=Material(diffuseColor=(0.5,0.5,0.5),shininess=1,specularColor=(0.3,0.3,0.3))),
              geometry=Sphere(radius=0.04))]),
          Transform(translation=(1.37,0.1,-0.1),
            children=[
            Shape(USE='Eye')]),
          Transform(scale=(2.6,1,0.7),translation=(1.46,0,0),
            #  Mouth 
            children=[
            Shape(USE='Eye')]),
          Transform(DEF='Left_Pectoral',translation=(1.1,-0.15,-0.1),
            children=[
            Shape(
              appearance=Appearance(USE='Fin_Skin'),
              geometry=IndexedFaceSet(coordIndex=[0,1,2,-1],solid=False,
                coord=Coordinate(point=[(0,0,0),(0.2,0,0),(-0.2,0,-0.2)])))]),
          Transform(DEF='Right_Pectoral',translation=(1.1,-0.15,0.1),
            children=[
            Shape(
              appearance=Appearance(USE='Fin_Skin'),
              geometry=IndexedFaceSet(coordIndex=[0,1,2,-1],solid=False,
                coord=Coordinate(point=[(0,0,0),(0.2,0,0),(-0.2,0,0.2)])))]),
          Group(DEF='AnimationGroup',
            children=[
            TimeSensor(DEF='Timer',cycleInterval=5,loop=True),
            CoordinateInterpolator(DEF='Waggler',key=[0.0,0.15,0.28,0.4,0.55,0.7,0.85,1],keyValue=[(1.5,0,0),(1.2,0,0),(0.5,0,0),(0.3,0,0),(0,0,0),(-0.3,0,0),(1.5,0,0),(1.20,0,0),(0.5,0,.1),(0.3,0,.2),(0,0,0.25),(-0.3,0,0.3),(1.5,0,0),(1.20,0,0),(0.5,0,0),(0.3,0,0),(0,0,0),(-0.3,0,0),(1.5,0,0),(1.2,0,0),(0.5,0,-0.1),(0.3,0,-0.2),(0,0,-0.25),(-0.3,0,-0.3),(1.5,0,0),(1.2,0,0),(0.5,0,0),(0.3,0,0),(0,0,0),(-0.3,0,0),(1.5,0,0),(1.2,0,0),(0.5,0,0.1),(0.3,0,0.2),(0,0,0.25),(-0.3,0,0.3),(1.5,0,0),(1.2,0,0),(0.5,0,0),(0.3,0,0),(0,0,0),(-0.3,0,0),(1.5,0,0),(1.2,0,0),(0.5,0,-0.1),(0.3,0,-0.2),(0,0,-0.25),(-0.3,0,-0.3)]),
            OrientationInterpolator(DEF='Left_Pectoral_Rotation',key=[0,0.3,0.7,1],keyValue=[(1,0,0,-0.52),(1,0,0,0),(1,0,0,0),(1,0,0,-0.52)]),
            OrientationInterpolator(DEF='Right_Pectoral_Rotation',key=[0,0.3,0.7,1],keyValue=[(1,0,0,-0.52),(1,0,0,0),(1,0,0,0),(1,0,0,-0.52)]),
            #  orbiting time 
            TimeSensor(DEF='OrbitClock',cycleInterval=90,enabled=False,loop=True),
            OrientationInterpolator(DEF='Orbit',key=[0.0,0.25,0.5,0.750,1],keyValue=[(0,1,0,6.28),(0,1,0,1.57),(0,1,0,3.14),(0,1,0,4.71),(0,1,0,6.28)]),
            PositionInterpolator(DEF='HalfMoonPath',key=[0,0.125,0.25,0.375,0.5,0.625,0.75,0.875,1],keyValue=[(15,2,0),(15,0,-10),(0,-1,-10),(-15,0,-10),(-15,2,0),(-15,0,10),(0,-2,10),(15,0,10),(15,2,0)]),
            ROUTE(fromField='fraction_changed',fromNode='Timer',toField='set_fraction',toNode='Waggler'),
            ROUTE(fromField='value_changed',fromNode='Waggler',toField='set_spine',toNode='Fish_Shape'),
            ROUTE(fromField='fraction_changed',fromNode='Timer',toField='set_fraction',toNode='Left_Pectoral_Rotation'),
            ROUTE(fromField='value_changed',fromNode='Left_Pectoral_Rotation',toField='set_rotation',toNode='Left_Pectoral'),
            ROUTE(fromField='fraction_changed',fromNode='Timer',toField='set_fraction',toNode='Right_Pectoral_Rotation'),
            ROUTE(fromField='value_changed',fromNode='Right_Pectoral_Rotation',toField='set_rotation',toNode='Right_Pectoral'),
            ROUTE(fromField='fraction_changed',fromNode='OrbitClock',toField='set_fraction',toNode='Orbit'),
            ROUTE(fromField='fraction_changed',fromNode='OrbitClock',toField='set_fraction',toNode='HalfMoonPath'),
            ROUTE(fromField='value_changed',fromNode='Orbit',toField='set_rotation',toNode='MyHalfMoon'),
            ROUTE(fromField='value_changed',fromNode='HalfMoonPath',toField='set_translation',toNode='MyHalfMoon')])])])),
    #  ==================== 
    ProtoInstance(name='HalfMoon')])
) # X3D model complete

####################################################################################################
# Self-test diagnostics
####################################################################################################

print('Self-test diagnostics for HalfMoonPrototype.py:')
if        metaDiagnostics(newModel): # built-in utility method in X3D class
    print(metaDiagnostics(newModel)) # display meta info, hint, warning, error, TODO values in this model
# print('check newModel.XML() serialization...')
newModelXML= newModel.XML() # test export method XML() for exceptions during export
newModel.XMLvalidate()
# print(newModelXML) # diagnostic

try:
#   print('check newModel.VRML() serialization...')
    newModelVRML=newModel.VRML() # test export method VRML() for exceptions during export
    # print(prependLineNumbers(newModelVRML)) # debug
    print("Python-to-VRML export of VRML output successful", flush=True)
except Exception as err: # usually BaseException
    # https://stackoverflow.com/questions/18176602/how-to-get-the-name-of-an-exception-that-was-caught-in-python
    print("*** Python-to-VRML export of VRML output failed:", type(err).__name__, err)
    if newModelVRML: # may have failed to generate
        print(prependLineNumbers(newModelVRML, err.lineno))

try:
#   print('check newModel.JSON() serialization...')
    newModelJSON=newModel.JSON() # test export method JSON() for exceptions during export
#   print(prependLineNumbers(newModelJSON)) # debug
    print("Python-to-JSON export of JSON output successful (under development)")
except Exception as err: # usually SyntaxError
    print("*** Python-to-JSON export of JSON output failed:", type(err).__name__, err)
    if newModelJSON: # may have failed to generate
        print(prependLineNumbers(newModelJSON,err.lineno))

print("python HalfMoonPrototype.py load and self-test diagnostics complete.")
